(() => {
  if (!window.inlayUpdateGivingInit) {
    // This is the first time this *type* of Inlay has been encountered.
    // We need to define anything global here.

    // Create the boot function.
    window.inlayUpdateGivingInit = async (inlay) => {

      const appDiv = document.createElement('div');
      appDiv.classList.add('inlay-update-giving');
      inlay.script.insertAdjacentElement('afterend', appDiv);
      CiviCRMInlay.debug("inlayUpdateGivingInit r3", inlay.script.dataset.inlayId);

      appDiv.innerHTML = `
        <div class=test-mode style="display:none;background-color: red; padding: 1em; color: black;text-align: center;">This form is undergoing maintenance, please come back later.</div>
        <h2>Update your giving</h2>
        <div class=public-intro></div>
        <div class=confirm-user>Hi, <strong class=name></strong>. <a href></a></div>
        <div class=current-giving>Loading...</div>
        <form class=change-form>
          <div class=change-amount>
            <label>New amount</label>
            <input type=number />
          </div>
          <div class=change-go>
            <button >Update giving amount</button>
          </div>
        </form>
      `;
      const currentGivingNode = appDiv.querySelector('.current-giving'),
        form = appDiv.querySelector('.change-form'),
        amountInput = appDiv.querySelector('.change-form input'),
        changeSubmitBtn = appDiv.querySelector('.change-form button'),
        confirmUser = appDiv.querySelector('.confirm-user'),
        confirmUserNot = confirmUser.querySelector('a'),
        publicIntro = appDiv.querySelector('.public-intro');

      let donatePageURL = inlay.initData.donatePageURL.replace(/#.*$/, '');
      let q = new URLSearchParams(window.location.search);
      ['source', 'medium', 'campaign', 'term', 'content'].forEach(p => {
        const utm = 'utm_' + p;
        if (q.has(utm)) {
          donatePageURL += (donatePageURL.indexOf('?') === -1 ? '?' : '&') + utm + '=' + encodeURIComponent(q.get(utm));
        }
      });
      CiviCRMInlay.debug("donatePageURL", donatePageURL);

      appDiv.querySelector('h2').textContent = inlay.initData.publicTitle;
      if (inlay.initData.testMode) {
        appDiv.querySelector('.test-mode').style.display = '';
        appDiv.style.border = 'solid 2px red';
        appDiv.style.padding = '1em';
      }
      if (inlay.initData.css) {
        const css = document.createElement('style');
        css.innerHTML = inlay.initData.css;
        document.head.appendChild(css);
      }
      publicIntro.innerHTML = inlay.initData.publicIntroHTML;
      form.style.opacity = 0;
      form.style.pointerEvents = 'none';
      form.style.transition = '0.3s all';
      confirmUserNot.href = donatePageURL;
      confirmUserNot.addEventListener('click', e => {
        e.preventDefault();
        e.stopPropagation();
        CiviCRMInlay.debug('User identified as not who we sent the link to...');
        window.location.replace(donatePageURL);
        return;
      });

      // Check checksum.
      const urlInput = (new URLSearchParams(window.location.hash.replace(/^#/, '').replaceAll('~', '&')));
      CiviCRMInlay.debug('location hash: ', window.location.hash.replace(/^#/, '').replaceAll('~', '&'), 'urlInput', urlInput);
      if (!urlInput || !urlInput.get('cs') || !urlInput.get('cid')) {
        // Checksum missing/wrong syntax just send to donate page.
        CiviCRMInlay.debug("Missing checksum");
        window.location.replace(donatePageURL);
        return;
      }
      const cs = urlInput.get('cs');
      const ctID = urlInput.get('cid');

      // Load data
      let d = await inlay.request({
        method: "post",
        body: {
          need: "prefillData",
          cs, ctID
        },
      }).catch(e => {
        CiviCRMInlay.debug("prefillData error", e);
      });
      if (!d.responseOk) {
        CiviCRMInlay.debug("Mismatched checksum");
        window.location.replace(donatePageURL);
        return;
      }
      CiviCRMInlay.debug('inlayUpdateGivingInit: prefill', d);
      if (d.descriptions.length > 1) {
        currentGivingNode.innerHTML = '<p>It looks like you may have several separate regular donations set up:</p><ul><li>'
          + d.descriptions.join('</li><li>')
          + '</li></ul><p>If you would like, you can consolidate these into a single direct debit instead.</p>';
      }
      else if (d.descriptions.length === 1) {
        currentGivingNode.innerHTML = '<p>' + d.descriptions[0] + '</p>';
      }
      else {
        // hmmm.
        const url = donatePageURL + `#cs=${cs}~cid=${ctID}`;
        CiviCRMInlay.debug('Hmmm, no regular donations found? Redirecting to donate.', url);
        window.location.replace(url);
        return;
      }

      confirmUserNot.textContent = "Not " + d.first_name + ' ' + d.last_name + '?';
      confirmUser.querySelector('.name').textContent = d.first_name;
      amountInput.value = d.suggested || '';
      if (d.updateDirectly) {
        // We are able to offer to change.
        changeSubmitBtn.textContent = 'Update giving amount';
      }
      else {
        changeSubmitBtn.textContent = 'Replace with a new Direct Debit';
      }
      form.style.opacity = '1';
      form.style.pointerEvents = 'all';

      form.addEventListener('submit', async e => {
        e.preventDefault();
        e.stopPropagation();
        const newAmount = parseFloat(amountInput.value);
        if (d.currentAmount && d.currentAmount == newAmount) {
          alert("Amount is the same as you are currently giving.");
          return;
        }
        if (newAmount < 1) {
          alert("Sorry, we can’t process amounts less than 1. If you need to cancel, please just get in touch.");
          return;
        }
        // ok.
        amountInput.disabled = 1;
        changeSubmitBtn.disabled = 1;
        form.style.opacity = '0.5';

        // Can we change it?
        if (d.suggested) {
          let result = await inlay.request({
            method: "post",
            body: {
              need: "change",
              sourceURL: window.location.href.replace(/#.*/, ''),
              cs,
              ctID,
              newAmount: amountInput.value,
            },
          });
          if (!result.success) {
            alert("Sorry, something we weren’t able to make the change. " + (result.error || ''));
            return;
          }
          CiviCRMInlay.debug({ result });
          form.style.display = 'none';
          publicIntro.style.display = 'none';
          confirmUser.style.display = 'none';
          currentGivingNode.innerHTML = result.summary + inlay.initData.thanksHTML;
        }
        else {
          // We're unable to make a simple change, send them to the form.
          CiviCRMInlay.debug('Sending user to new donation form as not simple.');
          const url = donatePageURL + `#cs=${cs}~cid=${ctID}~amt=${newAmount}`;
          window.location.replace(url);
          return;
        }
      });
    };
  }
})();
