<?php
namespace Civi;

trait UpdateRecurTraits {

  /** @var Array name => ContactID */
  protected array $contacts;

  /** @var Array contactID => recurID */
  protected array $recurs;

  /** Array of payment processors like { CLASSNAME: TEST_PP_ID } */
  protected $pps = [];

  /**
   * Populates properties: $this->contacts, $this->recurs.
   *
   * Requires $this->pps to be set if $paymentProcessorName is given.
   *
   * @var string $paymentProcessorName
   * @var int $n number of contacts & recurs pairs to create (1 - 3).
   */
  protected function createFixture(?string $paymentProcessorName = NULL, int $n = 3) :void {
    $contacts = array_slice([
           ['amount' => 1, 'display_name' => 'WilmaUpgrader'],
           ['amount' => 3, 'display_name' => 'BarneyDowngrader'],
           ['amount' => 2, 'display_name' => 'EmberMaintainer'],
         ], 0, $n);

    $records = array_map(function($_) { unset($_['amount']); return $_; }, $contacts);

    $this->contacts = \Civi\Api4\Contact::save(FALSE)
         ->setDefaults([
           'contact_type' => 'Individual',
         ])
         ->setRecords($contacts)
         ->execute()
         ->indexBy('display_name')
         ->column('id');

    $defaults = [
          'currency'                    => 'GBP',
          'contribution_status_id:name' => 'In Progress',
          'financial_type_id:name'      => 'Donation',
         ];
    if ($paymentProcessorName !== NULL) {
      $defaults['payment_processor_id'] = $this->pps[$paymentProcessorName];
    }
    $records = array_map(function($_) { return ['contact_id' => $this->contacts[$_['display_name']], 'amount' => $_['amount']]; }, $contacts);
    $this->recurs = \Civi\Api4\ContributionRecur::save(FALSE)
         ->setDefaults($defaults)
         ->setRecords($records)
         ->execute()
         ->indexBy('contact_id')
         ->column('id');

  }

}
